@@ -176,15 +176,6 @@ let convert_constant ctx p const =
 		| ConstFloat f | ConstDouble f -> Some (EConst (Float (Printf.sprintf "%E" f)), p)
 		| _ -> None) None const
 
-let rec same_sig parent jsig =
-	match jsig with
-	| TObject (p,targs) -> parent = p || List.exists (function | TType (_,s) -> same_sig parent s | _ -> false) targs
-	| TObjectInner(p, ntargs) ->
-			parent = (p, String.concat "$" (List.map fst ntargs)) ||
-			List.exists (fun (_,targs) -> List.exists (function | TType(_,s) -> same_sig parent s | _ -> false) targs) ntargs
-	| TArray(s,_) -> same_sig parent s
-	| _ -> false
-
 let convert_constraints ctx p tl = match tl with
 	| [] -> None
 	| [t] -> Some (convert_signature ctx p t,null_pos)
@@ -197,7 +188,6 @@ let convert_param ctx p parent param =
 		| (name, None, implemem_sig) ->
 			name, implemem_sig
 		in
-		let constraints = List.map (fun s -> if same_sig parent s then (TObject( (["java";"lang"], "Object"), [])) else s) constraints in
 		{
 			tp_name = jname_to_hx name,null_pos;
 			tp_params = [];
@@ -247,7 +237,7 @@ let convert_java_enum ctx p pe =
 		d_data = List.rev !data;
 	}
 
-	let convert_java_field ctx p jc field =
+	let convert_java_field ctx p jc is_interface field =
 		let p = { p with pfile =	p.pfile ^" (" ^field.jf_name ^")" } in
 		let cff_doc = None in
 		let cff_pos = p in
@@ -285,6 +275,8 @@ let convert_java_enum ctx p pe =
 			| JVolatile -> cff_meta := (Meta.Volatile, [], p) :: !cff_meta
 			| JTransient -> cff_meta := (Meta.Transient, [], p) :: !cff_meta
 			(* | JVarArgs -> cff_meta := (Meta.VarArgs, [], p) :: !cff_meta *)
+			| JAbstract when not is_interface ->
+				cff_access := (AAbstract, p) :: !cff_access
 			| _ -> ()
 		) field.jf_flags;
 
@@ -307,6 +299,34 @@ let convert_java_enum ctx p pe =
 				| _ -> ()
 		) field.jf_throws;
 
+		let extract_local_names () =
+			let default i =
+				"param" ^ string_of_int i
+			in
+			match field.jf_code with
+			| None ->
+				default
+			| Some attribs -> try
+				let rec loop attribs = match attribs with
+					| AttrLocalVariableTable locals :: _ ->
+						locals
+					| _ :: attribs ->
+						loop attribs
+					| [] ->
+						raise Not_found
+				in
+				let locals = loop attribs in
+				let h = Hashtbl.create 0 in
+				List.iter (fun local ->
+					Hashtbl.replace h local.ld_index local.ld_name
+				) locals;
+				(fun i ->
+					try Hashtbl.find h (i - 1) (* they are 1-based *)
+					with Not_found -> "param" ^ string_of_int i
+				)
+			with Not_found ->
+				default
+		in
 		let kind = match field.jf_kind with
 			| JKField when !readonly ->
 				FProp (("default",null_pos), ("null",null_pos), Some (convert_signature ctx p field.jf_signature,null_pos), None)
@@ -315,6 +335,7 @@ let convert_java_enum ctx p pe =
 			| JKMethod ->
 				match field.jf_signature with
 				| TMethod (args, ret) ->
+					let local_names = extract_local_names() in
 					let old_types = ctx.jtparams in
 					(match ctx.jtparams with
 					| c :: others -> ctx.jtparams <- (c @ field.jf_types) :: others
@@ -322,10 +343,10 @@ let convert_java_enum ctx p pe =
 					let i = ref 0 in
 					let args = List.map (fun s ->
 						incr i;
-						("param" ^ string_of_int !i,null_pos), false, [], Some(convert_signature ctx p s,null_pos), None
+						(local_names !i,null_pos), false, [], Some(convert_signature ctx p s,null_pos), None
 					) args in
 					let t = Option.map_default (convert_signature ctx p) (mk_type_path ctx ([], "Void") []) ret in
-					cff_meta := (Meta.Overload, [], p) :: !cff_meta;
+					cff_access := (AOverload,p) :: !cff_access;
 					let types = List.map (function
 						| (name, Some ext, impl) ->
 							{
@@ -423,16 +444,20 @@ let convert_java_enum ctx p pe =
 				meta := (Meta.LibType,[],p) :: !meta;
 
 			let is_interface = ref false in
+			let is_abstract =  ref false in
 			List.iter (fun f -> match f with
 				| JFinal -> flags := HFinal :: !flags
 				| JInterface ->
 						is_interface := true;
 						flags := HInterface :: !flags
-				| JAbstract -> meta := (Meta.Abstract, [], p) :: !meta
+				| JAbstract ->
+					meta := (Meta.Abstract, [], p) :: !meta;
+					is_abstract := true;
 				| JAnnotation -> meta := (Meta.Annotation, [], p) :: !meta
 				| _ -> ()
 			) jc.cflags;
 
+			if !is_abstract && not !is_interface then flags := HAbstract :: !flags;
 			(match jc.csuper with
 				| TObject( (["java";"lang"], "Object"), _ ) -> ()
 				| TObject( (["haxe";"lang"], "HxObject"), _ ) -> meta := (Meta.HxGen,[],p) :: !meta
@@ -458,7 +483,7 @@ let convert_java_enum ctx p pe =
 						if !is_interface && List.mem JStatic f.jf_flags then
 							()
 						else begin
-							fields := convert_java_field ctx p jc f :: !fields;
+							fields := convert_java_field ctx p jc !is_interface f :: !fields;
 							jfields := f :: !jfields
 						end
 					with
@@ -660,7 +685,7 @@ let compare_type com s1 s2 =
 				let implements = List.map (japply_params jparams) c.cinterfaces in
 				loop ~first_error:first_error super s2 || List.exists (fun super -> loop ~first_error:first_error super s2) implements
 			with | Not_found ->
-				print_endline ("-java-lib: The type " ^ (s_sig s1) ^ " is referred but was not found. Compilation may not occur correctly.");
+				print_endline ("--java-lib: The type " ^ (s_sig s1) ^ " is referred but was not found. Compilation may not occur correctly.");
 				print_endline "Did you forget to include a needed lib?";
 				if first_error then
 					not (loop ~first_error:false s2 s1)
@@ -1071,20 +1096,24 @@ class java_library_jar com name file_path = object(self)
 	val zip = lazy (Zip.open_in file_path)
 	val mutable cached_files = None
 	val cached_types = Hashtbl.create 12
+	val mutable loaded = false
 	val mutable closed = false
 
 	method load =
-		List.iter (function
-			| { Zip.is_directory = false; Zip.filename = filename } when String.ends_with filename ".class" ->
-				let pack = String.nsplit filename "/" in
-				(match List.rev pack with
-					| [] -> ()
-					| name :: pack ->
-						let name = String.sub name 0 (String.length name - 6) in
-						let pack = List.rev pack in
-						Hashtbl.add hxpack_to_jpack (jpath_to_hx (pack,name)) (pack,name))
-			| _ -> ()
-		) (Zip.entries (Lazy.force zip))
+		if not loaded then begin
+			loaded <- true;
+			List.iter (function
+				| { Zip.is_directory = false; Zip.filename = filename } when String.ends_with filename ".class" ->
+					let pack = String.nsplit filename "/" in
+					(match List.rev pack with
+						| [] -> ()
+						| name :: pack ->
+							let name = String.sub name 0 (String.length name - 6) in
+							let pack = List.rev pack in
+							Hashtbl.add hxpack_to_jpack (jpath_to_hx (pack,name)) (pack,name))
+				| _ -> ()
+			) (Zip.entries (Lazy.force zip))
+		end
 
 	method private lookup' ((pack,name) : path) : java_lib_type =
 		try
@@ -1101,17 +1130,18 @@ class java_library_jar com name file_path = object(self)
 		try
 			Hashtbl.find cached_types path
 		with | Not_found -> try
+			self#load;
 			let pack, name = self#convert_path path in
 			let try_file (pack,name) =
 				match self#lookup' (pack,name) with
 				| None ->
-						Hashtbl.add cached_types path None;
-						None
+					Hashtbl.add cached_types path None;
+					None
 				| Some (i, p1, p2) ->
-						Hashtbl.add cached_types path (Some(i,p1,p2)); (* type loop normalization *)
-						let ret = Some (normalize_jclass com i, p1, p2) in
-						Hashtbl.replace cached_types path ret;
-						ret
+					Hashtbl.add cached_types path (Some(i,p1,p2)); (* type loop normalization *)
+					let ret = Some (normalize_jclass com i, p1, p2) in
+					Hashtbl.replace cached_types path ret;
+					ret
 			in
 			try_file (pack,name)
 		with Not_found ->
@@ -1188,7 +1218,7 @@ class java_library_dir com name file_path = object(self)
 			| _ -> None
 end
 
-let add_java_lib com name std extern =
+let add_java_lib com name std extern modern =
 	let file = if Sys.file_exists name then
 		name
 	else try Common.find_file com name with
@@ -1196,11 +1226,14 @@ let add_java_lib com name std extern =
 		| Not_found ->
 			failwith ("Java lib " ^ name ^ " not found")
 	in
-	let java_lib = match (Unix.stat file).st_kind with
+	let java_lib =
+		if modern then
+			(new JavaModern.java_library_modern com name file :> (java_lib_type,unit) native_library)
+		else match (Unix.stat file).st_kind with
 		| S_DIR ->
-			(new java_library_dir com name file :> java_library)
+			(new java_library_dir com name file :> (java_lib_type,unit) native_library)
 		| _ ->
-			(new java_library_jar com name file :> java_library)
+			(new java_library_jar com name file :> (java_lib_type,unit) native_library)
 	in
 	if std then java_lib#add_flag FlagIsStd;
 	if extern then java_lib#add_flag FlagIsExtern;
