@@ -4,6 +4,7 @@ open TType
 
 let monomorph_create_ref : (unit -> tmono) ref = ref (fun _ -> die "" __LOC__)
 let monomorph_bind_ref : (tmono -> t -> unit) ref = ref (fun _ _ -> die "" __LOC__)
+let monomorph_classify_constraints_ref : (tmono -> tmono_constraint_kind) ref = ref (fun _ -> die "" __LOC__)
 
 let has_meta m ml = List.exists (fun (m2,_,_) -> m = m2) ml
 let get_meta m ml = List.find (fun (m2,_,_) -> m = m2) ml
@@ -19,6 +20,18 @@ let set_flag flags flag =
 let unset_flag flags flag =
 	flags land (lnot (1 lsl flag))
 
+let int_of_class_flag (flag : flag_tclass) =
+	Obj.magic flag
+
+let add_class_flag c (flag : flag_tclass) =
+	c.cl_flags <- set_flag c.cl_flags (int_of_class_flag flag)
+
+let remove_class_flag c (flag : flag_tclass) =
+	c.cl_flags <- unset_flag c.cl_flags (int_of_class_flag flag)
+
+let has_class_flag c (flag : flag_tclass) =
+	has_flag c.cl_flags (int_of_class_flag flag)
+
 let int_of_class_field_flag (flag : flag_tclass_field) =
 	Obj.magic flag
 
@@ -31,6 +44,18 @@ let remove_class_field_flag cf (flag : flag_tclass_field) =
 let has_class_field_flag cf (flag : flag_tclass_field) =
 	has_flag cf.cf_flags (int_of_class_field_flag flag)
 
+let int_of_var_flag (flag : flag_tvar) =
+	Obj.magic flag
+
+let add_var_flag v (flag : flag_tvar) =
+	v.v_flags <- set_flag v.v_flags (int_of_var_flag flag)
+
+let remove_var_flag v (flag : flag_tvar) =
+	v.v_flags <- unset_flag v.v_flags (int_of_var_flag flag)
+
+let has_var_flag v (flag : flag_tvar) =
+	has_flag v.v_flags (int_of_var_flag flag)
+
 (* ======= General utility ======= *)
 
 let alloc_var =
@@ -42,11 +67,10 @@ let alloc_var =
 			v_name = n;
 			v_type = t;
 			v_id = !uid;
-			v_capture = false;
-			v_final = (match kind with VUser TVOLocalFunction -> true | _ -> false);
 			v_extra = None;
 			v_meta = [];
-			v_pos = p
+			v_pos = p;
+			v_flags = (match kind with VUser TVOLocalFunction -> int_of_var_flag VFinal | _ -> 0);
 		}
 	)
 
@@ -93,9 +117,7 @@ let mk_class m path pos name_pos =
 		cl_meta = [];
 		cl_private = false;
 		cl_kind = KNormal;
-		cl_extern = false;
-		cl_final = false;
-		cl_interface = false;
+		cl_flags = 0;
 		cl_params = [];
 		cl_using = [];
 		cl_super = None;
@@ -108,24 +130,14 @@ let mk_class m path pos name_pos =
 		cl_array_access = None;
 		cl_constructor = None;
 		cl_init = None;
-		cl_overrides = [];
 		cl_build = (fun() -> Built);
 		cl_restore = (fun() -> ());
 		cl_descendants = [];
 	}
 
 let module_extra file sign time kind policy =
-	let file_key = ref None in
 	{
-		m_file = file;
-		m_file_key = (fun () ->
-			match !file_key with
-			| Some key -> key
-			| None ->
-				let key = Path.UniqueKey.create file in
-				file_key := Some key;
-				key
-		);
+		m_file = Path.UniqueKey.create_lazy file;
 		m_sign = sign;
 		m_display = {
 			m_inline_calls = [];
@@ -168,6 +180,7 @@ let null_module = {
 		m_id = alloc_mid();
 		m_path = [] , "";
 		m_types = [];
+		m_statics = None;
 		m_extra = module_extra "" "" 0. MFake [];
 	}
 
@@ -199,6 +212,7 @@ let null_abstract = {
 	a_array = [];
 	a_read = None;
 	a_write = None;
+	a_enum = false;
 }
 
 let add_dependency m mdep =
@@ -240,7 +254,7 @@ let map loop t =
 		(match r.tm_type with
 		| None -> t
 		| Some t -> loop t) (* erase*)
-	| TEnum (_,[]) | TInst (_,[]) | TType (_,[]) ->
+	| TEnum (_,[]) | TInst (_,[]) | TType (_,[]) | TAbstract (_,[]) ->
 		t
 	| TEnum (e,tl) ->
 		TEnum (e, List.map loop tl)
@@ -254,13 +268,7 @@ let map loop t =
 		TFun (List.map (fun (s,o,t) -> s, o, loop t) tl,loop r)
 	| TAnon a ->
 		let fields = PMap.map (fun f -> { f with cf_type = loop f.cf_type }) a.a_fields in
-		begin match !(a.a_status) with
-			| Opened ->
-				a.a_fields <- fields;
-				t
-			| _ ->
-				mk_anon ~fields a.a_status
-		end
+		mk_anon ~fields a.a_status
 	| TLazy f ->
 		let ft = lazy_type f in
 		let ft2 = loop ft in
@@ -268,6 +276,33 @@ let map loop t =
 	| TDynamic t2 ->
 		if t == t2 then	t else TDynamic (loop t2)
 
+let iter loop t =
+	match t with
+	| TMono r ->
+		(match r.tm_type with
+		| None -> ()
+		| Some t -> loop t)
+	| TEnum (_,[]) | TInst (_,[]) | TType (_,[]) ->
+		()
+	| TEnum (e,tl) ->
+		List.iter loop tl
+	| TInst (c,tl) ->
+		List.iter loop tl
+	| TType (t2,tl) ->
+		List.iter loop tl
+	| TAbstract (a,tl) ->
+		List.iter loop tl
+	| TFun (tl,r) ->
+		List.iter (fun (_,_,t) -> loop t) tl;
+		loop r
+	| TAnon a ->
+		PMap.iter (fun _ f -> loop f.cf_type) a.a_fields
+	| TLazy f ->
+		let ft = lazy_type f in
+		loop ft
+	| TDynamic t2 ->
+		if t != t2 then	loop t2
+
 let duplicate t =
 	let monos = ref [] in
 	let rec loop t =
@@ -384,13 +419,7 @@ let apply_params ?stack cparams params t =
 			TFun (List.map (fun (s,o,t) -> s, o, loop t) tl,loop r)
 		| TAnon a ->
 			let fields = PMap.map (fun f -> { f with cf_type = loop f.cf_type }) a.a_fields in
-			begin match !(a.a_status) with
-				| Opened ->
-					a.a_fields <- fields;
-					t
-				| _ ->
-					mk_anon ~fields a.a_status
-			end
+			mk_anon ~fields a.a_status
 		| TLazy f ->
 			let ft = lazy_type f in
 			let ft2 = loop ft in
@@ -463,28 +492,22 @@ let rec follow_without_null t =
 		follow_without_null (apply_params t.t_params tl t.t_type)
 	| _ -> t
 
-(** Assumes `follow` has already been applied *)
-let rec ambiguate_funs t =
+let rec follow_without_type t =
 	match t with
-	| TFun _ -> TFun ([], t_dynamic)
 	| TMono r ->
 		(match r.tm_type with
-		| Some _ -> die "" __LOC__
+		| Some t -> follow_without_type t
 		| _ -> t)
-	| TInst (a, pl) ->
-	    TInst (a, List.map ambiguate_funs pl)
-	| TEnum (a, pl) ->
-	    TEnum (a, List.map ambiguate_funs pl)
-	| TAbstract (a, pl) ->
-	    TAbstract (a, List.map ambiguate_funs pl)
-	| TType (a, pl) ->
-	    TType (a, List.map ambiguate_funs pl)
-	| TDynamic _ -> t
-	| TAnon a ->
-	    TAnon { a with a_fields =
-		    PMap.map (fun af -> { af with cf_type =
-				ambiguate_funs af.cf_type }) a.a_fields }
-	| TLazy _ -> die "" __LOC__
+	| TLazy f ->
+		follow_without_type (lazy_type f)
+	| TAbstract({a_path = [],"Null"},[t]) ->
+		follow_without_type t
+	| _ -> t
+
+let rec ambiguate_funs t =
+	match follow t with
+	| TFun _ -> TFun ([], t_dynamic)
+	| _ -> map ambiguate_funs t
 
 let rec is_nullable ?(no_lazy=false) = function
 	| TMono r ->
@@ -569,8 +592,6 @@ let concat e1 e2 =
 	) in
 	mk e e2.etype (punion e1.epos e2.epos)
 
-let is_closed a = !(a.a_status) <> Opened
-
 let type_of_module_type = function
 	| TClassDecl c -> TInst (c,List.map snd c.cl_params)
 	| TEnumDecl e -> TEnum (e,List.map snd e.e_params)
@@ -676,7 +697,7 @@ let rec raw_class_field build_type c tl i =
 			in
 			loop tl
 		| _ ->
-			if not c.cl_interface then raise Not_found;
+			if not (has_class_flag c CInterface) then raise Not_found;
 			(*
 				an interface can implements other interfaces without
 				having to redeclare its fields
@@ -728,20 +749,30 @@ let quick_field_dynamic t s =
 	try quick_field t s
 	with Not_found -> FDynamic s
 
-let rec get_constructor build_type c =
+let rec get_constructor_class c tl =
 	match c.cl_constructor, c.cl_super with
-	| Some c, _ -> build_type c, c
+	| Some cf, _ -> (cf,c,tl)
 	| None, None -> raise Not_found
-	| None, Some (csup,cparams) ->
-		let t, c = get_constructor build_type csup in
-		apply_params csup.cl_params cparams t, c
+	| None, Some (csup,tlsup) -> get_constructor_class csup (List.map (apply_params c.cl_params tl) tlsup)
+
+let rec get_constructor c =
+	match c.cl_constructor, c.cl_super with
+	| Some c, _ -> c
+	| None, None -> raise Not_found
+	| None, Some (csup,_) -> get_constructor csup
 
 let has_constructor c =
 	try
-		ignore(get_constructor (fun cf -> cf.cf_type) c);
+		ignore(get_constructor c);
 		true
 	with Not_found -> false
 
+let is_module_fields_class c =
+	match c.cl_kind with KModuleFields _ -> true | _ -> false
+
+let is_pos_outside_class c p =
+	p.pfile <> c.cl_pos.pfile || p.pmax < c.cl_pos.pmin || p.pmin > c.cl_pos.pmax
+
 let resolve_typedef t =
 	match t with
 	| TClassDecl _ | TEnumDecl _ | TAbstractDecl _ -> t
@@ -764,3 +795,9 @@ let type_has_meta t m =
 		| TType ({ t_meta = metadata }, _)
 		| TAbstract ({ a_meta = metadata }, _) -> has_meta m metadata
 
+(* tvar *)
+
+let var_extra params e = {
+	v_params = params;
+	v_expr = e;
+}
\ No newline at end of file
