@@ -243,7 +243,7 @@ let check_unification ctx e t =
 	begin match e.eexpr,t with
 		| TLocal v,TType({t_path = ["cs"],("Ref" | "Out")},_) ->
 			(* TODO: this smells of hack, but we have to deal with it somehow *)
-			v.v_capture <- true
+			add_var_flag v VCaptured;
 		| _ ->
 			()
 	end;
@@ -318,7 +318,7 @@ let save_class_state ctx t = match t with
 			List.fold_left (fun pmap f -> PMap.add f.cf_name f pmap) PMap.empty lst
 		in
 
-		let meta = c.cl_meta and path = c.cl_path and ext = c.cl_extern and over = c.cl_overrides in
+		let meta = c.cl_meta and path = c.cl_path and ext = (has_class_flag c CExtern) in
 		let sup = c.cl_super and impl = c.cl_implements in
 		let csr = Option.map (mk_field_restore) c.cl_constructor in
 		let ofr = List.map (mk_field_restore) c.cl_ordered_fields in
@@ -329,7 +329,7 @@ let save_class_state ctx t = match t with
 			c.cl_super <- sup;
 			c.cl_implements <- impl;
 			c.cl_meta <- meta;
-			c.cl_extern <- ext;
+			if ext then add_class_flag c CExtern else remove_class_flag c CExtern;
 			c.cl_path <- path;
 			c.cl_init <- init;
 			c.cl_ordered_fields <- List.map restore_field ofr;
@@ -337,7 +337,6 @@ let save_class_state ctx t = match t with
 			c.cl_fields <- mk_pmap c.cl_ordered_fields;
 			c.cl_statics <- mk_pmap c.cl_ordered_statics;
 			c.cl_constructor <- Option.map restore_field csr;
-			c.cl_overrides <- over;
 			c.cl_descendants <- [];
 			List.iter (fun (v, t) -> v.v_type <- t) !vars;
 		)
@@ -348,7 +347,7 @@ let save_class_state ctx t = match t with
 
 let remove_generic_base ctx t = match t with
 	| TClassDecl c when is_removable_class c ->
-		c.cl_extern <- true
+		add_class_flag c CExtern;
 	| _ ->
 		()
 
@@ -420,6 +419,26 @@ let apply_native_paths ctx t =
 			c.cl_meta <- meta :: c.cl_meta;
 			c.cl_path <- path;
 		| TEnumDecl e ->
+			let did_change = ref false in
+			let field _ ef = try
+				let meta,name = get_real_name ef.ef_meta ef.ef_name in
+				ef.ef_name <- name;
+				ef.ef_meta <- meta :: ef.ef_meta;
+				did_change := true;
+			with Not_found ->
+				()
+			in
+			PMap.iter field e.e_constrs;
+			if !did_change then begin
+				let names = ref [] in
+				e.e_constrs <- PMap.fold
+					(fun ef map ->
+						names := ef.ef_name :: !names;
+						PMap.add ef.ef_name ef map
+					)
+					e.e_constrs PMap.empty;
+				e.e_names <- !names;
+			end;
 			let meta,path = get_real_path e.e_meta e.e_path in
 			e.e_meta <- meta :: e.e_meta;
 			e.e_path <- path;
@@ -519,7 +538,7 @@ let add_meta_field ctx t = match t with
 				| Cs | Java -> false
 				| _ -> true
 			in
-			if c.cl_interface && not (can_deal_with_interface_metadata()) then begin
+			if (has_class_flag c CInterface) && not (can_deal_with_interface_metadata()) then begin
 				(* borrowed from gencommon, but I did wash my hands afterwards *)
 				let path = fst c.cl_path,snd c.cl_path ^ "_HxMeta" in
 				let ncls = mk_class c.cl_module path c.cl_pos null_pos in
@@ -540,7 +559,7 @@ let add_meta_field ctx t = match t with
 	this filter checks for their existence and also adds some metadata for analyzer and C# generator
 *)
 let check_cs_events com t = match t with
-	| TClassDecl cl when not cl.cl_extern ->
+	| TClassDecl cl when not (has_class_flag cl CExtern) ->
 		let check fields f =
 			match f.cf_kind with
 			| Var { v_read = AccNormal; v_write = AccNormal } when Meta.has Meta.Event f.cf_meta ->
@@ -605,7 +624,7 @@ let check_void_field ctx t = match t with
    This makes the first extended (implemented) interface the super for efficiency reasons (you can get one for 'free')
    and leaves the remaining ones as 'implemented' *)
 let promote_first_interface_to_super ctx t = match t with
-	| TClassDecl c when c.cl_interface ->
+	| TClassDecl c when (has_class_flag c CInterface) ->
 		begin match c.cl_implements with
 		| ({ cl_path = ["cpp";"rtti"],_ },_ ) :: _ -> ()
 		| first_interface  :: remaining ->
@@ -628,7 +647,7 @@ let check_reserved_type_paths ctx t =
 			ctx.com.warning ("Type path " ^ (s_type_path path) ^ " is reserved on this target") pos
 	in
 	match t with
-	| TClassDecl c when not c.cl_extern -> check c.cl_path c.cl_pos
+	| TClassDecl c when not (has_class_flag c CExtern) -> check c.cl_path c.cl_pos
 	| TEnumDecl e when not e.e_extern -> check e.e_path e.e_pos
 	| _ -> ()
 
@@ -642,7 +661,7 @@ let is_cached t =
 	m.m_processed <> !pp_counter
 
 let apply_filters_once ctx filters t =
-	if not (is_cached t) then run_expression_filters ctx filters t
+	if not (is_cached t) then run_expression_filters None ctx filters t
 
 let next_compilation() =
 	incr pp_counter
@@ -663,6 +682,10 @@ let iter_expressions fl mt =
 let filter_timer detailed s =
 	Timer.timer (if detailed then "filters" :: s else ["filters"])
 
+let timer_label detailed s =
+	if detailed then Some ("filters" :: s)
+	else None
+
 module ForRemap = struct
 	let apply ctx e =
 		let rec loop e = match e.eexpr with
@@ -673,7 +696,10 @@ module ForRemap = struct
 			let restore = save_locals ctx in
 			let e = ForLoop.IterationKind.to_texpr ctx v iterator e2 e.epos in
 			restore();
-			e
+			begin match e.eexpr with
+			| TFor _ -> for_remap ctx.com.basic v e1 e2 e.epos
+			| _ -> e
+			end
 		| _ ->
 			Type.map_expr loop e
 		in
@@ -709,22 +735,20 @@ let run com tctx main =
 	NullSafety.run com new_types;
 	(* PASS 1: general expression filters *)
 	let filters = [
-		ForRemap.apply tctx;
-		VarLazifier.apply com;
-		AbstractCast.handle_abstract_casts tctx;
+		"ForRemap",ForRemap.apply tctx;
+		"VarLazifier",VarLazifier.apply com;
+		"handle_abstract_casts",AbstractCast.handle_abstract_casts tctx;
 	] in
-	let t = filter_timer detail_times ["expr 0"] in
-	List.iter (run_expression_filters tctx filters) new_types;
-	t();
+	List.iter (run_expression_filters (timer_label detail_times ["expr 0"]) tctx filters) new_types;
 	let filters = [
-		fix_return_dynamic_from_void_function tctx true;
-		check_local_vars_init tctx.com;
-		check_abstract_as_value;
-		if defined com Define.AnalyzerOptimize then Tre.run tctx else (fun e -> e);
-		Optimizer.reduce_expression tctx;
-		if Common.defined com Define.OldConstructorInline then Optimizer.inline_constructors tctx else InlineConstructors.inline_constructors tctx;
-		Exceptions.filter tctx;
-		CapturedVars.captured_vars com;
+		"fix_return_dynamic_from_void_function",fix_return_dynamic_from_void_function tctx true;
+		"check_local_vars_init",check_local_vars_init tctx.com;
+		"check_abstract_as_value",check_abstract_as_value;
+		"Tre",if defined com Define.AnalyzerOptimize then Tre.run tctx else (fun e -> e);
+		"reduce_expression",Optimizer.reduce_expression tctx;
+		"inline_constructors",InlineConstructors.inline_constructors tctx;
+		"Exceptions_filter",Exceptions.filter tctx;
+		"captured_vars",CapturedVars.captured_vars com;
 	] in
 	let filters =
 		match com.platform with
@@ -736,9 +760,7 @@ let run com tctx main =
 			filters
 		| _ -> filters
 	in
-	let t = filter_timer detail_times ["expr 1"] in
-	List.iter (run_expression_filters tctx filters) new_types;
-	t();
+	List.iter (run_expression_filters (timer_label detail_times ["expr 1"]) tctx filters) new_types;
 	(* PASS 1.5: pre-analyzer type filters *)
 	let filters =
 		match com.platform with
@@ -762,16 +784,14 @@ let run com tctx main =
 	com.stage <- CAnalyzerDone;
 	let locals = RenameVars.init com in
 	let filters = [
-		Optimizer.sanitize com;
-		if com.config.pf_add_final_return then add_final_return else (fun e -> e);
-		(match com.platform with
+		"sanitize",Optimizer.sanitize com;
+		"add_final_return",if com.config.pf_add_final_return then add_final_return else (fun e -> e);
+		"RenameVars",(match com.platform with
 		| Eval -> (fun e -> e)
 		| _ -> RenameVars.run tctx locals);
-		mark_switch_break_loops;
+		"mark_switch_break_loops",mark_switch_break_loops;
 	] in
-	let t = filter_timer detail_times ["expr 2"] in
-	List.iter (run_expression_filters tctx filters) new_types;
-	t();
+	List.iter (run_expression_filters (timer_label detail_times ["expr 2"]) tctx filters) new_types;
 	next_compilation();
 	let t = filter_timer detail_times ["callbacks"] in
 	List.iter (fun f -> f()) (List.rev com.callbacks#get_before_save); (* macros onGenerate etc. *)
@@ -808,7 +828,13 @@ let run com tctx main =
 	t();
 	com.stage <- CDceDone;
 	(* PASS 3: type filters post-DCE *)
-	List.iter (run_expression_filters tctx [Exceptions.insert_save_stacks tctx]) new_types;
+	List.iter
+		(run_expression_filters
+			(timer_label detail_times [])
+			tctx
+			["insert_save_stacks",Exceptions.insert_save_stacks tctx]
+		)
+		new_types;
 	let type_filters = [
 		Exceptions.patch_constructors;
 		check_private_path;
