@@ -29,42 +29,13 @@ exception Continue
 exception Return of value
 exception Sys_exit of int
 
-let is v path =
-	if path = key_Dynamic then
-		v <> vnull
-	else match v with
-	| VInt32 _ -> path = key_Int || path = key_Float
-	| VFloat f -> path = key_Float || (path = key_Int && f = (float_of_int (int_of_float f)) && f <= 2147483647. && f >= -2147483648.)
-	| VTrue | VFalse -> path = key_Bool
-	| VPrototype {pkind = PClass _} -> path = key_Class
-	| VPrototype {pkind = PEnum _} -> path = key_Enum
-	| VEnumValue ve -> path = key_EnumValue || path = ve.epath
-	| VString _ -> path = key_String
-	| VArray _ -> path = key_Array
-	| VVector _ -> path = key_eval_Vector
-	| VInstance vi ->
-		let has_interface path' =
-			try begin match (get_static_prototype_raise (get_ctx()) path').pkind with
-				| PClass interfaces -> List.mem path interfaces
-				| _ -> false
-			end with Not_found ->
-				false
-		in
-		let rec loop proto =
-			if path = proto.ppath || has_interface proto.ppath then true
-			else begin match proto.pparent with
-				| Some proto -> loop proto
-				| None -> false
-			end
-		in
-		loop vi.iproto
-	| _ -> false
-
 let s_value_kind = function
 	| VNull -> "VNull"
 	| VTrue -> "VTrue"
 	| VFalse -> "VFalse"
 	| VInt32 _ -> "VInt32"
+	| VInt64 _ -> "VInt64"
+	| VUInt64 _ -> "VUInt64"
 	| VFloat _ -> "VFloat"
 	| VEnumValue _ -> "VEnumValue"
 	| VObject _ -> "VObject"
@@ -76,6 +47,8 @@ let s_value_kind = function
 	| VFunction _ -> "VFunction"
 	| VFieldClosure _ -> "VFieldClosure"
 	| VLazy _ -> "VLazy"
+	| VNativeString _ -> "VNativeString"
+	| VHandle _ -> "VHandle"
 
 let unexpected_value : 'a . value -> string -> 'a = fun v s ->
 	let str = match v with
@@ -127,13 +100,19 @@ let handle_stack_overflow eval f =
 	with Stack_overflow -> exc_string "Stack overflow"
 
 let catch_exceptions ctx ?(final=(fun() -> ())) f p =
-	let prev = !GlobalState.get_ctx_ref in
+	let reset_ctx =
+		if !GlobalState.initialized then
+			let prev = !GlobalState.get_ctx_ref in
+			(fun() -> GlobalState.get_ctx_ref := prev)
+		else
+			(fun() -> ())
+	in
 	select ctx;
 	let eval = get_eval ctx in
 	let env = eval.env in
 	let r = try
 		let v = handle_stack_overflow eval f in
-		GlobalState.get_ctx_ref := prev;
+		reset_ctx();
 		final();
 		Some v
 	with
@@ -144,7 +123,7 @@ let catch_exceptions ctx ?(final=(fun() -> ())) f p =
 		if is v key_haxe_macro_Error then begin
 			let v1 = field v key_exception_message in
 			let v2 = field v key_pos in
-			GlobalState.get_ctx_ref := prev;
+			reset_ctx();
 			final();
 			match v1 with
 				| VString s ->
@@ -175,7 +154,7 @@ let catch_exceptions ctx ?(final=(fun() -> ())) f p =
 				| _ :: l -> l (* Otherwise, ignore topmost frame position. *)
 			in
 			let msg = get_exc_error_message ctx v stack (if p' = null_pos then p else p') in
-			GlobalState.get_ctx_ref := prev;
+			reset_ctx();
 			final();
 			Error.error msg null_pos
 		end
@@ -183,7 +162,7 @@ let catch_exceptions ctx ?(final=(fun() -> ())) f p =
 		final();
 		None
 	| exc ->
-		GlobalState.get_ctx_ref := prev;
+		reset_ctx();
 		final();
 		raise exc
 	in
