@@ -43,12 +43,16 @@ let key_max = hash "max"
 let key_file = hash "file"
 let key_len = hash "len"
 let key_message = hash "message"
+let key_name = hash "name"
+let key_error = hash "error"
 let key_exception_message = hash "__exceptionMessage"
 let key_native_exception = hash "__nativeException"
 let key_native_stack = hash "__nativeStack"
 let key_Array = hash "Array"
 let key_eval_Vector = hash "eval.Vector"
 let key_String = hash "String"
+let key_haxe_Exception = hash "haxe.Exception"
+let key_haxe_ds_Option = hash "haxe.ds.Option"
 let key_haxe_ds_StringMap = hash "haxe.ds.StringMap"
 let key_haxe_ds_IntMap = hash "haxe.ds.IntMap"
 let key_haxe_ds_ObjectMap = hash "haxe.ds.ObjectMap"
@@ -136,6 +140,7 @@ let key_sys_net_Mutex = hash "sys.thread.Mutex"
 let key_sys_net_Lock = hash "sys.thread.Lock"
 let key_sys_net_Tls = hash "sys.thread.Tls"
 let key_sys_net_Deque = hash "sys.thread.Deque"
+let key_sys_thread_EventLoop = hash "sys.thread.EventLoop"
 
 let key_mbedtls_Config = hash "mbedtls.Config"
 let key_mbedtls_CtrDrbg = hash "mbedtls.CtrDrbg"
@@ -143,3 +148,58 @@ let key_mbedtls_Entropy = hash "mbedtls.Entropy"
 let key_mbedtls_PkContext = hash "mbedtls.PkContext"
 let key_mbedtls_Ssl = hash "mbedtls.Ssl"
 let key_mbedtls_X509Crt = hash "mbedtls.X509Crt"
+
+let key_eval_luv_Result = hash "eval.luv.Result"
+let key_eval_luv_LuvException = hash "eval.luv.LuvException"
+let key_eval_luv_ReceiveHandle = hash "eval.luv.ReceiveHandle"
+let key_eval_luv_AddressFamily = hash "eval.luv.AddressFamily"
+let key_eval_luv_SocketType = hash "eval.luv.SocketType"
+let key_onExit = hash "onExit"
+let key_environment = hash "environment"
+let key_redirect = hash "redirect"
+let key_workingDirectory = hash "workingDirectory"
+let key_uid = hash "uid"
+let key_gid = hash "gid"
+let key_windowsVerbatimArguments = hash "windowsVerbatimArguments"
+let key_detached = hash "detached"
+let key_windowsHide = hash "windowsHide"
+let key_windowsHideConsole = hash "windowsHideConsole"
+let key_windowsHideGui = hash "windowsHideGui"
+let key_request = hash "request"
+let key_family = hash "family"
+let key_sockType = hash "sockType"
+let key_protocol = hash "protocol"
+let key_flags = hash "flags"
+let key_addr = hash "addr"
+let key_data = hash "data"
+let key_canonName = hash "canonName"
+let key_node = hash "node"
+let key_service = hash "service"
+let key_sec = hash "sec"
+let key_nsec = hash "nsec"
+let key_usec = hash "usec"
+let key_blksize = hash "blksize"
+let key_blocks = hash "blocks"
+let key_gen = hash "gen"
+let key_atim = hash "atim"
+let key_mtim = hash "mtim"
+let key_ctim = hash "ctim"
+let key_birthtim = hash "birthtim"
+let key_width = hash "width"
+let key_height = hash "height"
+let key_type = hash "type"
+let key_bsize = hash "bsize"
+let key_bfree = hash "bfree"
+let key_bavail = hash "bavail"
+let key_files = hash "files"
+let key_ffree = hash "ffree"
+let key_fspare = hash "fspare"
+let key_kind = hash "kind"
+let key_end = hash "end"
+let key_events = hash "events"
+let key_isInternal = hash "isInternal"
+let key_physical = hash "physical"
+let key_address = hash "address"
+let key_netmask = hash "netmask"
+let key_previous = hash "previous"
+let key_current = hash "current"
\ No newline at end of file
