@@ -62,7 +62,7 @@ open EvalJitContext
 let rec op_assign ctx jit e1 e2 = match e1.eexpr with
 	| TLocal var ->
 		let exec = jit_expr jit false e2 in
-		if var.v_capture then emit_capture_write (get_capture_slot jit var) exec
+		if has_var_flag var VCaptured then emit_capture_write (get_capture_slot jit var) exec
 		else emit_local_write (get_slot jit var.v_id e1.epos) exec
 	| TField(ef,fa) ->
 		let name = hash (field_name fa) in
@@ -74,7 +74,7 @@ let rec op_assign ctx jit e1 e2 = match e1.eexpr with
 			| FStatic({cl_path=path},_) | FEnum({e_path=path},_) ->
 				let proto = get_static_prototype jit.ctx (path_hash path) ef.epos in
 				emit_proto_field_write proto (get_proto_field_index proto name) exec2
-			| FInstance(c,_,_) when not c.cl_interface ->
+			| FInstance(c,_,_) when not (has_class_flag c CInterface) ->
 				let proto = get_instance_prototype jit.ctx (path_hash c.cl_path) ef.epos in
 				let i = get_instance_field_index proto name ef.epos in
 				emit_instance_field_write exec1 ef.epos i exec2
@@ -111,7 +111,7 @@ let rec op_assign ctx jit e1 e2 = match e1.eexpr with
 and op_assign_op jit op e1 e2 prefix = match e1.eexpr with
 	| TLocal var ->
 		let exec = jit_expr jit false e2 in
-		if var.v_capture then emit_capture_read_write (get_capture_slot jit var) exec op prefix
+		if has_var_flag var VCaptured then emit_capture_read_write (get_capture_slot jit var) exec op prefix
 		else emit_local_read_write (get_slot jit var.v_id e1.epos) exec op prefix
 	| TField(ef,fa) ->
 		let name = hash (field_name fa) in
@@ -121,7 +121,7 @@ and op_assign_op jit op e1 e2 prefix = match e1.eexpr with
 			| FStatic({cl_path=path},_) ->
 				let proto = get_static_prototype jit.ctx (path_hash path) ef.epos in
 				emit_proto_field_read_write proto (get_proto_field_index proto name) exec2 op prefix
-			| FInstance(c,_,_) when not c.cl_interface ->
+			| FInstance(c,_,_) when not (has_class_flag c CInterface) ->
 				let proto = get_instance_prototype jit.ctx (path_hash c.cl_path) ef.epos in
 				let i = get_instance_field_index proto name ef.epos in
 				emit_instance_field_read_write exec1 ef.epos i exec2 op prefix
@@ -163,7 +163,7 @@ and unop jit op flag e1 p =
 		emit_op_sub p (fun _ -> vint32 (Int32.minus_one)) exec
 	| Increment ->
 		begin match Texpr.skip e1 with
-		| {eexpr = TLocal v} when not v.v_capture ->
+		| {eexpr = TLocal v} when not (has_var_flag v VCaptured) ->
 			let slot = get_slot jit v.v_id e1.epos in
 			if flag = Prefix then emit_local_incr_prefix slot e1.epos
 			else emit_local_incr_postfix slot e1.epos
@@ -172,6 +172,10 @@ and unop jit op flag e1 p =
 		end
 	| Decrement ->
 		op_decr jit e1 (flag = Prefix) p
+	| Spread ->
+		match flag with
+		| Postfix -> die ~p:p "Postfix spread operator is not supported" __LOC__
+		| Prefix -> jit_expr jit false e1
 
 and jit_default jit return def =
 	match def with
@@ -410,7 +414,7 @@ and jit_expr jit return e =
 			let name = hash (field_name fa) in
 			let execs = List.map (jit_expr jit false) el in
 			let is_final c cf =
-				c.cl_final || (has_class_field_flag cf CfFinal) ||
+				has_class_flag c CFinal || (has_class_field_flag cf CfFinal) ||
 				(* In interp mode we can assume that a field is final if it is not overridden.
 				   We cannot do that in macro mode because overriding fields might be added
 				   after jitting this call. *)
@@ -441,6 +445,11 @@ and jit_expr jit return e =
 						| [exec1;exec2] -> emit_string_cca exec1 exec2 e.epos
 						| _ -> die "" __LOC__
 					end
+				| FStatic({cl_path=[],"StringTools"},{cf_name="unsafeCodeAt"}) ->
+					begin match execs with
+						| [exec1;exec2] -> emit_string_cca_unsafe exec1 exec2 e.epos
+						| _ -> die "" __LOC__
+					end
 				| FEnum({e_path=path},ef) ->
 					let key = path_hash path in
 					let pos = Some e.epos in
@@ -452,11 +461,11 @@ and jit_expr jit return e =
 				| FInstance(c,_,cf) when is_proper_method cf ->
 					if not (is_final c cf) then
 						default()
-					else if not c.cl_interface then
+					else if not (has_class_flag c CInterface) then
 						instance_call c
 					(* If we have exactly one implementer, use it instead of the super class/interface. *)
 					else if not ctx.is_macro && c.cl_implements = [] && c.cl_super = None then begin match c.cl_descendants with
-						| [c'] when not c'.cl_interface && is_final c' cf ->
+						| [c'] when not (has_class_flag c' CInterface) && is_final c' cf ->
 							instance_call c'
 						| _ ->
 							default()
@@ -517,7 +526,7 @@ and jit_expr jit return e =
 		end
 	(* read *)
 	| TLocal var ->
-		if var.v_capture then emit_capture_read (get_capture_slot jit var)
+		if has_var_flag var VCaptured then emit_capture_read (get_capture_slot jit var)
 		else emit_local_read (get_slot jit var.v_id e.epos)
 	| TField(e1,fa) ->
 		let name = hash (field_name fa) in
@@ -529,7 +538,7 @@ and jit_expr jit return e =
 			| FInstance({cl_path=path},_,{cf_kind = Method (MethNormal | MethInline)}) ->
 				let proto = get_static_prototype ctx (path_hash path) e1.epos in
 				emit_proto_field_read proto (get_proto_field_index proto name)
-			| FInstance(c,_,_) when not c.cl_interface ->
+			| FInstance(c,_,_) when not (has_class_flag c CInterface) ->
 				let proto = get_instance_prototype ctx (path_hash c.cl_path) e1.epos in
 				let i = get_instance_field_index proto name e1.epos in
 				begin match e1.eexpr with
@@ -685,25 +694,29 @@ and jit_tfunction jit static pos tf =
 	fl,exec
 
 and get_env_creation jit static file info =
-	create_env_info static file info jit.capture_infos jit.max_num_locals (Hashtbl.length jit.captures)
+	create_env_info static file (jit.ctx.file_keys#get file) info jit.capture_infos jit.max_num_locals (Hashtbl.length jit.captures)
+
+let jit_timer ctx f =
+	Std.finally (Timer.timer [(if ctx.is_macro then "macro" else "interp");"jit"]) f ()
 
 (* Creates a [EvalValue.vfunc] of function [tf], which can be [static] or not. *)
 let jit_tfunction ctx key_type key_field tf static pos =
-	let t = Timer.timer [(if ctx.is_macro then "macro" else "interp");"jit"] in
-	(* Create a new JitContext with an initial scope *)
-	let jit = EvalJitContext.create ctx in
-	let fl,exec = jit_tfunction jit static pos tf in
-	(* Create the [vfunc] instance depending on the number of arguments. *)
-	let hasret = jit.has_nonfinal_return in
-	let eci = get_env_creation jit static tf.tf_expr.epos.pfile (EKMethod(key_type,key_field)) in
-	let f = if hasret then create_function ctx eci exec fl else create_function_noret ctx eci exec fl in
-	t();
-	f
+	let f () =
+		(* Create a new JitContext with an initial scope *)
+		let jit = EvalJitContext.create ctx in
+		let fl,exec = jit_tfunction jit static pos tf in
+		(* Create the [vfunc] instance depending on the number of arguments. *)
+		let hasret = jit.has_nonfinal_return in
+		let eci = get_env_creation jit static tf.tf_expr.epos.pfile (EKMethod(key_type,key_field)) in
+		if hasret then create_function ctx eci exec fl else create_function_noret ctx eci exec fl
+	in
+	jit_timer ctx f
 
 (* JITs expression [e] to a function. This is used for expressions that are not in a method. *)
 let jit_expr ctx e =
-	let t = Timer.timer [(if ctx.is_macro then "macro" else "interp");"jit"] in
-	let jit = EvalJitContext.create ctx in
-	let f = jit_expr jit false (mk_block e) in
-	t();
-	jit,f
\ No newline at end of file
+	let f () =
+		let jit = EvalJitContext.create ctx in
+		let f = jit_expr jit false (mk_block e) in
+		jit,f
+	in
+	jit_timer ctx f
\ No newline at end of file
