@@ -39,7 +39,7 @@ let is_generic_parameter ctx c =
 	(* first check field parameters, then class parameters *)
 	try
 		ignore (List.assoc (snd c.cl_path) ctx.curfield.cf_params);
-		Meta.has Meta.Generic ctx.curfield.cf_meta
+		has_class_field_flag ctx.curfield CfGeneric
 	with Not_found -> try
 		ignore(List.assoc (snd c.cl_path) ctx.type_params);
 		(match ctx.curclass.cl_kind with | KGeneric -> true | _ -> false);
@@ -151,7 +151,7 @@ let get_native_name meta =
 let check_native_name_override ctx child base =
 	let error base_pos child_pos =
 		display_error ctx ("Field " ^ child.cf_name ^ " has different @:native value than in superclass") child_pos;
-		display_error ctx ("Base field is defined here") base_pos
+		display_error ctx (compl_msg "Base field is defined here") base_pos
 	in
 	try
 		let child_name, child_pos = get_native_name child.cf_meta in
@@ -166,25 +166,33 @@ let check_native_name_override ctx child base =
 let check_overriding ctx c f =
 	match c.cl_super with
 	| None ->
-		if List.memq f c.cl_overrides then display_error ctx ("Field " ^ f.cf_name ^ " is declared 'override' but doesn't override any field") f.cf_pos
-	| _ when c.cl_extern && Meta.has Meta.CsNative c.cl_meta -> () (* -net-lib specific: do not check overrides on extern CsNative classes *)
+		if has_class_field_flag f CfOverride then display_error ctx ("Field " ^ f.cf_name ^ " is declared 'override' but doesn't override any field") f.cf_pos
+	| _ when (has_class_flag c CExtern) && Meta.has Meta.CsNative c.cl_meta -> () (* -net-lib specific: do not check overrides on extern CsNative classes *)
 	| Some (csup,params) ->
 		let p = f.cf_name_pos in
 		let i = f.cf_name in
 		let check_field f get_super_field is_overload = try
-			(if is_overload && not (Meta.has Meta.Overload f.cf_meta) then
-				display_error ctx ("Missing @:overload declaration for field " ^ i) p);
+			(if is_overload && not (has_class_field_flag f CfOverload) then
+				display_error ctx ("Missing overload declaration for field " ^ i) p);
+			let f_has_override = has_class_field_flag f CfOverride in
 			let t, f2 = get_super_field csup i in
 			check_native_name_override ctx f f2;
 			(* allow to define fields that are not defined for this platform version in superclass *)
 			(match f2.cf_kind with
 			| Var { v_read = AccRequire _ } -> raise Not_found;
 			| _ -> ());
-			if ctx.com.config.pf_overload && (Meta.has Meta.Overload f2.cf_meta && not (Meta.has Meta.Overload f.cf_meta)) then
-				display_error ctx ("Field " ^ i ^ " should be declared with @:overload since it was already declared as @:overload in superclass") p
-			else if not (List.memq f c.cl_overrides) then
-				display_error ctx ("Field " ^ i ^ " should be declared with 'override' since it is inherited from superclass " ^ s_type_path csup.cl_path) p
-			else if not (has_class_field_flag f CfPublic) && (has_class_field_flag f2 CfPublic) then
+			if has_class_field_flag f2 CfAbstract then begin
+				if f_has_override then
+					display_error ctx ("Field " ^ i ^ " is declared 'override' but parent field " ^ i ^ " is 'abstract' and does not provide any implementation to override") p
+				else
+					add_class_field_flag f CfOverride (* our spec requires users to not "override" abstract functions, but our implementation depends on implementations to be declared with "override" ¯\_(ツ)_/¯ *)
+			end;
+			if (has_class_field_flag f2 CfOverload && not (has_class_field_flag f CfOverload)) then
+				display_error ctx ("Field " ^ i ^ " should be declared with overload since it was already declared as overload in superclass") p
+			else if not f_has_override && not (has_class_field_flag f2 CfAbstract) then begin
+				if has_class_flag c CExtern then add_class_field_flag f CfOverride
+				else display_error ctx ("Field " ^ i ^ " should be declared with 'override' since it is inherited from superclass " ^ s_type_path csup.cl_path) p
+			end else if not (has_class_field_flag f CfPublic) && (has_class_field_flag f2 CfPublic) then
 				display_error ctx ("Field " ^ i ^ " has less visibility (public/private) than superclass one") p
 			else (match f.cf_kind, f2.cf_kind with
 			| _, Method MethInline ->
@@ -201,11 +209,11 @@ let check_overriding ctx c f =
 			with
 				Unify_error l ->
 					display_error ctx ("Field " ^ i ^ " overrides parent class with different or incomplete type") p;
-					display_error ctx ("Base field is defined here") f2.cf_name_pos;
-					display_error ctx (error_msg (Unify l)) p;
+					display_error ctx (compl_msg "Base field is defined here") f2.cf_name_pos;
+					display_error ctx (compl_msg (error_msg (Unify l))) p;
 		with
 			Not_found ->
-				if List.memq f c.cl_overrides then
+				if has_class_field_flag f CfOverride then
 					let msg = if is_overload then
 						("Field " ^ i ^ " is declared 'override' but no compatible overload was found")
 					else begin
@@ -218,8 +226,8 @@ let check_overriding ctx c f =
 					end in
 					display_error ctx msg p
 		in
-		if ctx.com.config.pf_overload && Meta.has Meta.Overload f.cf_meta then begin
-			let overloads = Overloads.get_overloads csup i in
+		if has_class_field_flag f CfOverload then begin
+			let overloads = Overloads.get_overloads ctx.com csup i in
 			List.iter (fun (t,f2) ->
 				(* check if any super class fields are vars *)
 				match f2.cf_kind with
@@ -325,7 +333,7 @@ module Inheritance = struct
 
 	let check_extends ctx c t p = match follow t with
 		| TInst (csup,params) ->
-			if is_basic_class_path csup.cl_path && not (c.cl_extern && csup.cl_extern) then error "Cannot extend basic class" p;
+			if is_basic_class_path csup.cl_path && not ((has_class_flag c CExtern) && (has_class_flag csup CExtern)) then error "Cannot extend basic class" p;
 			if extends csup c then error "Recursive class" p;
 			begin match csup.cl_kind with
 				| KTypeParameter _ ->
@@ -335,69 +343,147 @@ module Inheritance = struct
 			end
 		| _ -> error "Should extend by using a class" p
 
-	let rec check_interface ctx c intf params =
+	let rec check_interface ctx missing c intf params =
+		List.iter (fun (i2,p2) ->
+			check_interface ctx missing c i2 (List.map (apply_params intf.cl_params params) p2)
+		) intf.cl_implements;
 		let p = c.cl_name_pos in
 		let rec check_field i f =
-			(if ctx.com.config.pf_overload then
-				List.iter (function
-					| f2 when f != f2 ->
-							check_field i f2
-					| _ -> ()) f.cf_overloads);
+			let t = (apply_params intf.cl_params params f.cf_type) in
 			let is_overload = ref false in
 			try
 				let t2, f2 = class_field_no_interf c i in
 				let t2, f2 =
-					if ctx.com.config.pf_overload && (f2.cf_overloads <> [] || Meta.has Meta.Overload f2.cf_meta) then
-						let overloads = Overloads.get_overloads c i in
+					if f2.cf_overloads <> [] || has_class_field_flag f2 CfOverload then
+						let overloads = Overloads.get_overloads ctx.com c i in
 						is_overload := true;
-						let t = (apply_params intf.cl_params params f.cf_type) in
 						List.find (fun (t1,f1) -> Overloads.same_overload_args t t1 f f1) overloads
 					else
 						t2, f2
 				in
-				ignore(follow f2.cf_type); (* force evaluation *)
-				let p = f2.cf_name_pos in
-				let mkind = function
-					| MethNormal | MethInline -> 0
-					| MethDynamic -> 1
-					| MethMacro -> 2
-				in
-				if (has_class_field_flag f CfPublic) && not (has_class_field_flag f2 CfPublic) && not (Meta.has Meta.CompilerGenerated f.cf_meta) then
-					display_error ctx ("Field " ^ i ^ " should be public as requested by " ^ s_type_path intf.cl_path) p
-				else if not (unify_kind f2.cf_kind f.cf_kind) || not (match f.cf_kind, f2.cf_kind with Var _ , Var _ -> true | Method m1, Method m2 -> mkind m1 = mkind m2 | _ -> false) then
-					display_error ctx ("Field " ^ i ^ " has different property access than in " ^ s_type_path intf.cl_path ^ " (" ^ s_kind f2.cf_kind ^ " should be " ^ s_kind f.cf_kind ^ ")") p
-				else try
-					valid_redefinition ctx f2 t2 f (apply_params intf.cl_params params f.cf_type)
-				with
-					Unify_error l ->
-						if not (Meta.has Meta.CsNative c.cl_meta && c.cl_extern) then begin
-							display_error ctx ("Field " ^ i ^ " has different type than in " ^ s_type_path intf.cl_path) p;
-							display_error ctx ("Interface field is defined here") f.cf_pos;
-							display_error ctx (error_msg (Unify l)) p;
-						end
-			with
-				| Not_found when not c.cl_interface ->
-					let msg = if !is_overload then
-						let ctx = print_context() in
-						let args = match follow f.cf_type with | TFun(args,_) -> String.concat ", " (List.map (fun (n,o,t) -> (if o then "?" else "") ^ n ^ " : " ^ (s_type ctx t)) args) | _ -> die "" __LOC__ in
-						"No suitable overload for " ^ i ^ "( " ^ args ^ " ), as needed by " ^ s_type_path intf.cl_path ^ " was found"
-					else
-						("Field " ^ i ^ " needed by " ^ s_type_path intf.cl_path ^ " is missing")
+				delay ctx PForce (fun () ->
+					ignore(follow f2.cf_type); (* force evaluation *)
+					let p = f2.cf_name_pos in
+					let mkind = function
+						| MethNormal | MethInline -> 0
+						| MethDynamic -> 1
+						| MethMacro -> 2
 					in
-					display_error ctx msg p
+					if (has_class_field_flag f CfPublic) && not (has_class_field_flag f2 CfPublic) && not (Meta.has Meta.CompilerGenerated f.cf_meta) then
+						display_error ctx ("Field " ^ i ^ " should be public as requested by " ^ s_type_path intf.cl_path) p
+					else if not (unify_kind f2.cf_kind f.cf_kind) || not (match f.cf_kind, f2.cf_kind with Var _ , Var _ -> true | Method m1, Method m2 -> mkind m1 = mkind m2 | _ -> false) then
+						display_error ctx ("Field " ^ i ^ " has different property access than in " ^ s_type_path intf.cl_path ^ " (" ^ s_kind f2.cf_kind ^ " should be " ^ s_kind f.cf_kind ^ ")") p
+					else try
+						valid_redefinition ctx f2 t2 f (apply_params intf.cl_params params f.cf_type)
+					with
+						Unify_error l ->
+							if not (Meta.has Meta.CsNative c.cl_meta && (has_class_flag c CExtern)) then begin
+								display_error ctx ("Field " ^ i ^ " has different type than in " ^ s_type_path intf.cl_path) p;
+								display_error ctx (compl_msg "Interface field is defined here") f.cf_pos;
+								display_error ctx (compl_msg (error_msg (Unify l))) p;
+							end
+				)
+			with
+				| Not_found when (has_class_flag c CAbstract) ->
+					let cf = {f with cf_overloads = []} in
+					add_class_field_flag cf CfAbstract;
+					begin try
+						let cf' = PMap.find cf.cf_name c.cl_fields in
+						Hashtbl.remove ctx.com.overload_cache (c.cl_path,i);
+						cf'.cf_overloads <- cf :: cf'.cf_overloads
+					with Not_found ->
+						TClass.add_field c cf
+					end
+				| Not_found when not (has_class_flag c CInterface) ->
+					if Diagnostics.is_diagnostics_run ctx.com c.cl_pos then
+						DynArray.add missing (f,t)
+					else begin
+						let msg = if !is_overload then
+							let ctx = print_context() in
+							let args = match follow f.cf_type with | TFun(args,_) -> String.concat ", " (List.map (fun (n,o,t) -> (if o then "?" else "") ^ n ^ " : " ^ (s_type ctx t)) args) | _ -> die "" __LOC__ in
+							"No suitable overload for " ^ i ^ "( " ^ args ^ " ), as needed by " ^ s_type_path intf.cl_path ^ " was found"
+						else
+							("Field " ^ i ^ " needed by " ^ s_type_path intf.cl_path ^ " is missing")
+						in
+						display_error ctx msg p
+					end
 				| Not_found -> ()
 		in
-		PMap.iter check_field intf.cl_fields;
-		List.iter (fun (i2,p2) ->
-			check_interface ctx c i2 (List.map (apply_params intf.cl_params params) p2)
-		) intf.cl_implements
+		let check_field i cf =
+			check_field i cf;
+			if has_class_field_flag cf CfOverload then
+				List.iter (check_field i) (List.rev cf.cf_overloads)
+		in
+		PMap.iter check_field intf.cl_fields
 
 	let check_interfaces ctx c =
 		match c.cl_path with
 		| "Proxy" :: _ , _ -> ()
-		| _ when c.cl_extern && Meta.has Meta.CsNative c.cl_meta -> ()
+		| _ when (has_class_flag c CExtern) && Meta.has Meta.CsNative c.cl_meta -> ()
 		| _ ->
-		List.iter (fun (intf,params) -> check_interface ctx c intf params) c.cl_implements
+		List.iter (fun (intf,params) ->
+			let missing = DynArray.create () in
+			check_interface ctx missing c intf params;
+			if DynArray.length missing > 0 then begin
+				let l = DynArray.to_list missing in
+				let diag = {
+					mf_pos = c.cl_name_pos;
+					mf_on = TClassDecl c;
+					mf_fields = List.map (fun (cf,t) -> (cf,t,CompletionType.from_type (Display.get_import_status ctx) t)) l;
+					mf_cause = ImplementedInterface(intf,params);
+				} in
+				let display = ctx.com.display_information in
+				display.module_diagnostics <- MissingFields diag :: display.module_diagnostics
+			end
+		) c.cl_implements
+
+	let check_abstract_class ctx c csup params =
+		let missing = ref [] in
+		let map = apply_params csup.cl_params params in
+		let check_abstract_class_field cf1 t1 =
+			try
+				let cf2 = PMap.find cf1.cf_name c.cl_fields in
+				if not (List.exists (fun cf2 ->
+					Overloads.same_overload_args t1 cf2.cf_type cf1 cf2
+				) (cf2 :: cf2.cf_overloads)) then
+					missing := (cf1,t1) :: !missing
+			with Not_found ->
+				missing := (cf1,t1) :: !missing
+		in
+		let cfl = TClass.get_all_fields csup params in
+		PMap.iter (fun _ (_,cf) ->
+			let cfl = Overloads.collect_overloads map csup cf.cf_name in
+			List.iter (fun (t,cf) ->
+				if (has_class_field_flag cf CfAbstract) then
+					check_abstract_class_field cf t
+			) cfl
+		) cfl;
+		match !missing with
+		| [] ->
+			()
+		| l when Diagnostics.is_diagnostics_run ctx.com c.cl_pos ->
+			let diag = {
+				mf_pos = c.cl_name_pos;
+				mf_on = TClassDecl c;
+				mf_fields = List.rev_map (fun (cf,t) -> (cf,t,CompletionType.from_type (Display.get_import_status ctx) t)) l;
+				mf_cause = AbstractParent(csup,params);
+			} in
+			let display = ctx.com.display_information in
+			display.module_diagnostics <- MissingFields diag :: display.module_diagnostics
+		| l ->
+			let singular = match l with [_] -> true | _ -> false in
+			display_error ctx (Printf.sprintf "This class extends abstract class %s but doesn't implement the following method%s" (s_type_path csup.cl_path) (if singular then "" else "s")) c.cl_name_pos;
+			display_error ctx (Printf.sprintf "Implement %s or make %s abstract as well" (if singular then "it" else "them") (s_type_path c.cl_path)) c.cl_name_pos;
+			let pctx = print_context() in
+			List.iter (fun (cf,_) ->
+				let s = match follow cf.cf_type with
+					| TFun(tl,tr) ->
+						String.concat ", " (List.map (fun (n,o,t) -> Printf.sprintf "%s:%s" n (s_type pctx t)) tl)
+					| t ->
+						s_type pctx t
+				in
+				display_error ctx (Printf.sprintf "... %s(%s)" cf.cf_name s) cf.cf_name_pos
+			) (List.rev !missing)
 
 	let set_heritance ctx c herits p =
 		let is_lib = Meta.has Meta.LibType c.cl_meta in
@@ -409,8 +495,8 @@ module Inheritance = struct
 				| Meta.AutoBuild, el, p -> c.cl_meta <- (Meta.Build,el,{ c.cl_pos with pmax = c.cl_pos.pmin }(* prevent display metadata *)) :: m :: c.cl_meta
 				| _ -> ()
 			) csup.cl_meta;
-			if csup.cl_final && not ((csup.cl_extern && Meta.has Meta.Hack c.cl_meta) || (match c.cl_kind with KTypeParameter _ -> true | _ -> false)) then
-				error ("Cannot extend a final " ^ if c.cl_interface then "interface" else "class") p;
+			if has_class_flag csup CFinal && not (((has_class_flag csup CExtern) && Meta.has Meta.Hack c.cl_meta) || (match c.cl_kind with KTypeParameter _ -> true | _ -> false)) then
+				error ("Cannot extend a final " ^ if (has_class_flag c CInterface) then "interface" else "class") p;
 		in
 		let check_cancel_build csup =
 			match csup.cl_build() with
@@ -454,15 +540,15 @@ module Inheritance = struct
 			if is_extends then begin
 				if c.cl_super <> None then error "Cannot extend several classes" p;
 				let csup,params = check_extends ctx c t p in
-				if c.cl_interface then begin
-					if not csup.cl_interface then error "Cannot extend by using a class" p;
+				if (has_class_flag c CInterface) then begin
+					if not (has_class_flag csup CInterface) then error "Cannot extend by using a class" p;
 					c.cl_implements <- (csup,params) :: c.cl_implements;
 					if not !has_interf then begin
-						if not is_lib then delay ctx PForce (fun() -> check_interfaces ctx c);
+						if not is_lib then delay ctx PConnectField (fun() -> check_interfaces ctx c);
 						has_interf := true;
 					end
 				end else begin
-					if csup.cl_interface then error "Cannot extend by using an interface" p;
+					if (has_class_flag csup CInterface) then error "Cannot extend by using an interface" p;
 					c.cl_super <- Some (csup,params)
 				end;
 				(fun () ->
@@ -470,17 +556,17 @@ module Inheritance = struct
 					process_meta csup;
 				)
 			end else begin match follow t with
-				| TInst ({ cl_path = [],"ArrayAccess"; cl_extern = true; },[t]) ->
+				| TInst ({ cl_path = [],"ArrayAccess" } as ca,[t]) when (has_class_flag ca CExtern) ->
 					if c.cl_array_access <> None then error "Duplicate array access" p;
 					c.cl_array_access <- Some t;
 					(fun () -> ())
 				| TInst (intf,params) ->
 					if extends intf c then error "Recursive class" p;
-					if c.cl_interface then error "Interfaces cannot implement another interface (use extends instead)" p;
-					if not intf.cl_interface then error "You can only implement an interface" p;
+					if (has_class_flag c CInterface) then error "Interfaces cannot implement another interface (use extends instead)" p;
+					if not (has_class_flag intf CInterface) then error "You can only implement an interface" p;
 					c.cl_implements <- (intf, params) :: c.cl_implements;
 					if not !has_interf && not is_lib && not (Meta.has (Meta.Custom "$do_not_check_interf") c.cl_meta) then begin
-						delay ctx PForce (fun() -> check_interfaces ctx c);
+						delay ctx PConnectField (fun() -> check_interfaces ctx c);
 						has_interf := true;
 					end;
 					(fun () ->
@@ -489,7 +575,7 @@ module Inheritance = struct
 					)
 				| TDynamic t ->
 					if c.cl_dynamic <> None then error "Cannot have several dynamics" p;
-					if not c.cl_extern then display_error ctx "In haxe 4, implements Dynamic is only supported on externs" p;
+					if not (has_class_flag c CExtern) then display_error ctx "In haxe 4, implements Dynamic is only supported on externs" p;
 					c.cl_dynamic <- Some t;
 					(fun () -> ())
 				| _ ->
@@ -502,20 +588,20 @@ module Inheritance = struct
 					Typeload.load_instance ~allow_display:true ctx (ct,p) false
 				with DisplayException(DisplayFields Some({fkind = CRTypeHint} as r)) ->
 					(* We don't allow `implements` on interfaces. Just raise fields completion with no fields. *)
-					if not is_extends && c.cl_interface then raise_fields [] CRImplements r.fsubject;
+					if not is_extends && (has_class_flag c CInterface) then raise_fields [] CRImplements r.fsubject;
 					let l = List.filter (fun item -> match item.ci_kind with
-						| ITType({kind = Interface} as cm,_) -> (not is_extends || c.cl_interface) && CompletionModuleType.get_path cm <> c.cl_path
+						| ITType({kind = Interface} as cm,_) -> (not is_extends || (has_class_flag c CInterface)) && CompletionModuleType.get_path cm <> c.cl_path
 						| ITType({kind = Class} as cm,_) ->
-							is_extends && not c.cl_interface && CompletionModuleType.get_path cm <> c.cl_path &&
+							is_extends && not (has_class_flag c CInterface) && CompletionModuleType.get_path cm <> c.cl_path &&
 							(not cm.is_final || Meta.has Meta.Hack c.cl_meta) &&
-							(not (is_basic_class_path (cm.pack,cm.name)) || (c.cl_extern && cm.is_extern))
+							(not (is_basic_class_path (cm.pack,cm.name)) || ((has_class_flag c CExtern) && cm.is_extern))
 						| _ -> false
 					) r.fitems in
 					raise_fields l (if is_extends then CRExtends else CRImplements) r.fsubject
 				in
 				Some (check_herit t is_extends p)
 			with Error(Module_not_found(([],name)),p) when ctx.com.display.dms_kind <> DMNone ->
-				if Diagnostics.is_diagnostics_run p then DisplayToplevel.handle_unresolved_identifier ctx name p true;
+				if Diagnostics.is_diagnostics_run ctx.com p then DisplayToplevel.handle_unresolved_identifier ctx name p true;
 				None
 		) herits in
 		fl
